@@ -1,13 +1,25 @@
-{-# LANGUAGE FlexibleInstances, TypeFamilies
-  , TemplateHaskell, DeriveDataTypeable #-}
-{-# OPTIONS_GHC -fno-warn-missing-signatures -fno-warn-incomplete-patterns -fno-warn-name-shadowing #-} 
--- we have lots of parsers which don't want signatures; and we have uniplate patterns
+{-# LANGUAGE DeriveFoldable    #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE TypeFamilies      #-}
 
--- Copyright (c) Anders Karlsson 2009
--- Copyright (c) JP Bernardy 2009
+-- we have lots of parsers which don't want signatures; and we have
+-- uniplate patterns
+{-# OPTIONS_GHC -fno-warn-missing-signatures
+                -fno-warn-incomplete-patterns
+                -fno-warn-name-shadowing #-}
+{-# OPTIONS_HADDOCK show-extensions #-}
+
+-- |
+-- Module      :  Yi.Syntax.Haskell
+-- License     :  GPL-2
+-- Maintainer  :  yi-devel@googlegroups.com
+-- Stability   :  experimental
+-- Portability :  portable
+--
 -- NOTES:
 -- Note if the layout of the first line (not comments)
 -- is wrong the parser will only parse what is in the blocks given by Layout.hs
+
 module Yi.Syntax.Haskell ( PModule
                          , PModuleDecl
                          , PImport
@@ -17,35 +29,32 @@ module Yi.Syntax.Haskell ( PModule
                          , indentScanner
                          ) where
 
-import Prelude ()
-import Data.Maybe
-import Data.List ((\\))
-import qualified Data.Foldable
-import Yi.IncrementalParse
-import Yi.Lexer.Alex
-import Yi.Lexer.Haskell
-import Yi.Syntax.Layout
-import Yi.Syntax.Tree
-import Yi.Syntax
-import Yi.Prelude
-import Prelude ()
-import Data.DeriveTH
-import Data.Tuple (uncurry)
-import Control.Arrow ((&&&))
+import           Control.Applicative (Alternative ((<|>), empty, many, some),
+                                      Applicative (..), optional, (<$>))
+import           Control.Arrow       ((&&&))
+import           Data.Foldable       (Foldable)
+import           Data.List           ((\\))
+import           Data.Maybe          (fromJust, isNothing)
+import           Yi.IncrementalParse
+import           Yi.Lexer.Alex       (Posn (Posn, posnOfs), Tok (Tok, tokT),
+                                      startPosn, tokBegin)
+import           Yi.Lexer.Haskell
+import           Yi.Syntax           (Scanner)
+import           Yi.Syntax.Layout    (State, layoutHandler)
+import           Yi.Syntax.Tree      (IsTree (emptyNode, uniplate), sepBy1)
 
-indentScanner :: Scanner (AlexState lexState) (TT)
-              -> Scanner (Yi.Syntax.Layout.State Token lexState) (TT)
+indentScanner :: Scanner (AlexState lexState) TT
+              -> Scanner (Yi.Syntax.Layout.State Token lexState) TT
 indentScanner = layoutHandler startsLayout [(Special '(', Special ')'),
                                             (Reserved Let, Reserved In),
                                             (Special '[', Special ']'),
                                             (Special '{', Special '}')]
                          ignoredToken
-                         ((Special '<'), (Special '>'), (Special '.'))
+                         (Special '<', Special '>', Special '.')
                          isBrace
 
--- HACK: We insert the Special '<', '>', '.',
--- which do not occur in normal haskell
--- parsing.
+-- HACK: We insert the Special '<', '>', '.', which do not occur in
+-- normal haskell parsing.
 
 -- | Check if a token is a brace, this function is used to
 -- fix the layout so that do { works correctly
@@ -54,7 +63,7 @@ isBrace (Tok br _ _) = Special '{' == br
 
 -- | Theese are the tokens ignored by the layout handler.
 ignoredToken :: TT -> Bool
-ignoredToken (Tok t _ (Posn _ _ _)) = isComment t || t == CppDirective
+ignoredToken (Tok t _ (Posn{})) = isComment t || t == CppDirective
 
 type Tree = PModule
 type PAtom = Exp
@@ -68,44 +77,44 @@ type PImport = Exp
 -- | Exp can be expression or declaration
 data Exp t
     = PModule { comments :: [t]
-              , progMod  :: (Maybe (PModule t))
+              , progMod  :: Maybe (PModule t)
               }
-    | ProgMod { modDecl :: (PModuleDecl t)
-              , body    :: (PModule t)  -- ^ The module declaration part
+    | ProgMod { modDecl :: PModuleDecl t
+              , body    :: PModule t  -- ^ The module declaration part
               }
     | Body { imports :: Exp t -- [PImport t]
-           , content :: (Block t)
-           , extraContent :: (Block t) -- ^ The body of the module
+           , content :: Block t
+           , extraContent :: Block t -- ^ The body of the module
            }
-    | PModuleDecl { moduleKeyword :: (PAtom t)
-                                 , name          :: (PAtom t)
-                                 , exports       :: (Exp t)
-                                 , whereKeyword  :: (Exp t)
-                                    }    
-    | PImport { importKeyword :: (PAtom t)
-                         , qual          :: (Exp t)
-                         , name'         :: (PAtom t)
-                         , as            :: (Exp t)
-                         , specification :: (Exp t)
+    | PModuleDecl { moduleKeyword :: PAtom t
+                                 , name          :: PAtom t
+                                 , exports       :: Exp t
+                                 , whereKeyword  :: Exp t
+                                    }
+    | PImport { importKeyword :: PAtom t
+                         , qual          :: Exp t
+                         , name'         :: PAtom t
+                         , as            :: Exp t
+                         , specification :: Exp t
                          }
 
-    | TS t [Exp t] -- ^ Type signature 
-    | PType { typeKeyword :: (PAtom t)
-            , typeCons    :: (Exp t)
-            , equal       :: (PAtom t)
-            , btype       :: (Exp t)
+    | TS t [Exp t] -- ^ Type signature
+    | PType { typeKeyword :: PAtom t
+            , typeCons    :: Exp t
+            , equal       :: PAtom t
+            , btype       :: Exp t
             } -- ^ Type declaration
-    | PData { dataKeyword :: (PAtom t)
-            , dtypeCons   :: (Exp t)
-            , dEqual      :: (Exp t)
-            , dataRhs     :: (Exp t)
+    | PData { dataKeyword :: PAtom t
+            , dtypeCons   :: Exp t
+            , dEqual      :: Exp t
+            , dataRhs     :: Exp t
             }  -- ^ Data declaration
-    | PData' { dEqual    :: (PAtom t)
-             , dataCons  :: (Exp t) -- ^ Data declaration RHS
+    | PData' { dEqual     :: PAtom t
+             , dataCons   :: Exp t -- ^ Data declaration RHS
              }
-    | PClass { cKeyword :: (PAtom t) -- Can be class or instance
-             , cHead        :: (Exp t)
-             , cwhere       :: (Exp t) -- ^ Class declaration
+    | PClass { cKeyword   :: PAtom t -- Can be class or instance
+             , cHead      :: Exp t
+             , cwhere     :: Exp t -- ^ Class declaration
              }
       -- declaration
       -- declarations and parts of them follow
@@ -137,16 +146,16 @@ data Exp t
     | DC (Exp t) -- ^ Data constructor
     | PLet (PAtom t) (Exp t) (Exp t) -- ^ let expression
     | PIn t [Exp t]
-  deriving Show
+  deriving (Show, Foldable)
 
-$(derive makeFoldable ''Exp)
 instance IsTree Exp where
    emptyNode = Expr []
    uniplate tree = case tree of
        (ProgMod a b)     -> ([a,b], \[a,b] -> ProgMod a b)
        (Body x exp exp') -> ([x, exp, exp'], \[x, exp, exp'] -> Body x exp exp')
        (PModule x (Just e)) -> ([e],\[e] -> PModule x (Just e))
-       (Paren l g r)  -> (l:g ++ [r], \(l:gr) -> Paren l (init gr) (last gr)) -- TODO: improve
+       (Paren l g r)  -> -- TODO: improve
+         (l:g ++ [r], \(l:gr) -> Paren l (init gr) (last gr))
        (RHS l g)      -> ([l,g],\[l,g] -> (RHS l g))
        (Block s)      -> (s,Block)
        (PLet l s i)   -> ([l,s,i],\[l,s,i] -> PLet l s i)
@@ -166,7 +175,7 @@ instance IsTree Exp where
        (DC e) -> ([e],\[e] -> DC e)
        PModuleDecl a b c d -> ([a,b,c,d],\[a,b,c,d] -> PModuleDecl a b c d)
        PImport a b c d e -> ([a,b,c,d,e],\[a,b,c,d,e] -> PImport a b c d e)
-       t              -> ([],\_->t)
+       t              -> ([],const t)
 
 -- | The parser
 parse :: P TT (Tree TT)
@@ -191,7 +200,7 @@ pModBody = (exact [startBlock] *>
        <|> (exact [nextLine] *> pBody)
        <|> Body <$> pure emptyNode <*> pEmptyBL <*> pEmptyBL
     where pBod  = Block <$> pBlocks pTopDecl
-          elems = [(Special ';'), nextLine, startBlock]
+          elems = [Special ';', nextLine, startBlock]
 
 -- | @pEmptyBL@ A parser returning an empty block
 pEmptyBL :: Parser TT (Exp TT)
@@ -212,11 +221,11 @@ noImports = notNext [Reserved Import] *> pure emptyNode
 -- Helper functions for parsing follows
 -- | Parse Variables
 pVarId :: Parser TT (Exp TT)
-pVarId = pAtom [VarIdent, (Reserved Other), (Reserved As)]
+pVarId = pAtom [VarIdent, Reserved Other, Reserved As]
 
 -- | Parse VarIdent and ConsIdent
 pQvarid :: Parser TT (Exp TT)
-pQvarid = pAtom [VarIdent, ConsIdent, (Reserved Other), (Reserved As)]
+pQvarid = pAtom [VarIdent, ConsIdent, Reserved Other, Reserved As]
 
 -- | Parse an operator using please
 pQvarsym :: Parser TT (Exp TT)
@@ -370,8 +379,8 @@ pImports :: Parser TT (Exp TT) -- [PImport TT]
 pImports = Expr <$> many (pImport
                  <* pTestTok pEol
                  <* optional (some $ exact [nextLine, Special ';']))
-        where pEol = [(Special ';'), nextLine, endBlock]
- 
+        where pEol = [Special ';', nextLine, endBlock]
+
 -- | Parse one import
 pImport :: Parser TT (PImport TT)
 pImport = PImport  <$> pAtom [Reserved Import]
@@ -387,14 +396,15 @@ pImport = PImport  <$> pAtom [Reserved Import]
               pExp'    = Bin
                      <$> (PAtom <$> sym
                           (uncurry (||) . (&&&)
-                           (flip elem [VarIdent, ConsIdent])
+                           (`elem` [VarIdent, ConsIdent])
                            isOperator) <*> pComments
                           <|>  pQvarsym)
                      <*> pOpt pImpS
 
 -- | Parse simple type synonyms
 pType :: Parser TT (Exp TT)
-pType = PType <$> (Bin <$> pAtom [Reserved Type] <*> pOpt (pAtom [Reserved Instance]))
+pType = PType <$> (Bin <$> pAtom [Reserved Type]
+                   <*> pOpt (pAtom [Reserved Instance]))
      <*> (TC . Expr <$> pTypeExpr')
      <*> ppAtom [ReservedOp Equal]
      <*> (TC . Expr <$> pTypeExpr')
@@ -424,10 +434,14 @@ pAtype = pAtype'
      <|> pErr
 
 pAtype' :: Parser TT (Exp TT)
-pAtype' = pQvarid
+pAtype' = pTypeCons
       <|> pParen (many $ pExprElem [])
       <|> pBrack (many $ pExprElem [])
 
+pTypeCons :: Parser TT (Exp TT)
+pTypeCons = Bin <$> pAtom [ConsIdent]
+            <*> please (pMany $ pAtom [VarIdent, ConsIdent])
+
 pContext :: Parser TT (Exp TT)
 pContext = Context <$> pOpt pForAll
        <*> (TC <$> (pClass' <|> pParenSep pClass'))
@@ -478,11 +492,14 @@ pLet = PLet <$> pAtom [Reserved Let]
 -- | Parse a Do block
 pDo :: Parser TT (Exp TT)
 pDo = Bin <$> pAtom [Reserved Do]
-          <*> pBlock (pExpr ((Special ';' : recognizedSometimes) \\ [ReservedOp LeftArrow]))
+          <*> pBlock (pExpr ((Special ';' : recognizedSometimes)
+                             \\ [ReservedOp LeftArrow]))
 
 -- | Parse part of a lambda binding.
 pLambda :: Parser TT (Exp TT)
-pLambda = Bin <$> pAtom [ReservedOp BackSlash] <*> (Bin <$> (Expr <$> pPattern) <*> please (pBareAtom [ReservedOp RightArrow]))
+pLambda = Bin <$> pAtom [ReservedOp BackSlash]
+          <*> (Bin <$> (Expr <$> pPattern)
+               <*> please (pBareAtom [ReservedOp RightArrow]))
 
 -- | Parse an Of block
 pOf :: Parser TT (Exp TT)
@@ -498,39 +515,49 @@ pAlternative = Bin <$> (Expr <$> pPattern)
 pClass :: Parser TT (Exp TT)
 pClass = PClass <$> pAtom [Reserved Class, Reserved Instance]
                 <*> (TC . Expr <$>  pTypeExpr')
-                <*> please (pWhere pTopDecl)
+                <*> pOpt (please (pWhere pTopDecl))
                 -- use topDecl since we have associated types and such.
-                      
+
 
 -- | Parse some guards and a where clause
 pGuard :: Token -> Parser TT (Exp TT)
 pGuard equalSign = PGuard
      <$> some (PGuard' <$> pCAtom [ReservedOp Pipe] pEmpty <*>
                -- comments are by default parsed after this
-               pExpr (recognizedSometimes \\ [ReservedOp LeftArrow, Special ',']) -- those two symbols can appear in guards.
+               pExpr (recognizedSometimes
+                      -- these two symbols can appear in guards.
+                      \\ [ReservedOp LeftArrow, Special ','])
                <*> please (pEq equalSign))
                -- this must be -> if used in case
 
 -- | Right-hand-side of a function or case equation (after the pattern)
 pFunRHS :: Token -> Parser TT (Exp TT)
-pFunRHS equalSign = Bin <$> (pGuard equalSign <|> pEq equalSign) <*> pOpt (pWhere pFunDecl)
+pFunRHS equalSign =
+  Bin <$> (pGuard equalSign <|> pEq equalSign) <*> pOpt (pWhere pFunDecl)
 
 pWhere :: Parser TT (Exp TT) -> Parser TT (Exp TT)
-pWhere p = PWhere <$> pAtom [Reserved Where] <*> please (pBlock p) <*> (pMany pErr)
+pWhere p =
+  PWhere <$> pAtom [Reserved Where] <*> please (pBlock p) <*> pMany pErr
 -- After a where there might "misaligned" code that do not "belong" to anything.
 -- Here we swallow it as errors.
 
 -- Note that this can both parse an equation and a type declaration.
 -- Since they can start with the same token, the left part is factored here.
 pDecl :: Bool -> Bool -> Parser TT (Exp TT)
-pDecl acceptType acceptEqu = Expr <$> ((Yuck $ Enter "missing end of type or equation declaration" $ pure [])
-             <|> ((:) <$> pElem False recognizedSometimes <*> pToList (pDecl acceptType acceptEqu))
-             <|> ((:) <$> pBareAtom [Special ','] <*> pToList (pDecl acceptType False))
-                 -- if a comma is found, then the rest must be a type declaration.
-             <|> (if acceptType then pTypeEnding else empty)
-             <|> (if acceptEqu  then pEquEnding else empty))
-    where pTypeEnding = ((:) <$> (TS <$> exact [ReservedOp DoubleColon] <*> pTypeExpr') <*> pure [])
-          pEquEnding = ((:) <$> pFunRHS (ReservedOp Equal) <*> pure [])
+pDecl acceptType acceptEqu =
+  Expr <$> ((Yuck $
+               Enter "missing end of type or equation declaration" $ pure [])
+            <|> ((:) <$> pElem False recognizedSometimes
+                 <*> pToList (pDecl acceptType acceptEqu))
+            <|> ((:) <$> pBareAtom [Special ',']
+                 <*> pToList (pDecl acceptType False))
+                 -- if a comma is found, then the rest must be a type
+                 -- declaration.
+            <|> (if acceptType then pTypeEnding else empty)
+            <|> (if acceptEqu  then pEquEnding else empty))
+    where pTypeEnding = (:) <$> (TS <$> exact [ReservedOp DoubleColon]
+                                 <*> pTypeExpr') <*> pure []
+          pEquEnding =  (:) <$> pFunRHS (ReservedOp Equal) <*> pure []
 
 pFunDecl = pDecl True True
 pTypeDecl = pDecl True False
@@ -559,9 +586,9 @@ pBlockOf p  = Block <$> pBlockOf' (pBlocks p) -- see HACK above
 
 
 pBlock :: Parser TT (Exp TT) -> Parser TT (Exp TT)
-pBlock p = pBlockOf' (Block <$> pBlocks' p) 
+pBlock p = pBlockOf' (Block <$> pBlocks' p)
        <|> pEBrace (p `sepBy1` exact [Special ';'] <|> pure [])
-       <|> (Yuck $ Enter "block expected" $ pEmptyBL)
+       <|> (Yuck $ Enter "block expected" pEmptyBL)
 
 -- | Parse something surrounded by (Special '<') and (Special '>')
 pBlockOf' :: Parser TT a -> Parser TT a
@@ -571,7 +598,7 @@ pBlockOf' p = exact [startBlock] *> p <* exact [endBlock] -- see HACK above
 
 -- | Parse something that can contain a data, type declaration or a class
 pTopDecl :: Parser TT (Exp TT)
-pTopDecl =    pFunDecl 
+pTopDecl =    pFunDecl
           <|> pType
           <|> pData
           <|> pClass
@@ -581,8 +608,8 @@ pTopDecl =    pFunDecl
 -- | A "normal" expression, where none of the following symbols are acceptable.
 pExpr' = pExpr recognizedSometimes
 
-recognizedSometimes = [ReservedOp DoubleDot, 
-                       Special ',', 
+recognizedSometimes = [ReservedOp DoubleDot,
+                       Special ',',
                        ReservedOp Pipe,
                        ReservedOp Equal,
                        ReservedOp LeftArrow,
@@ -598,10 +625,13 @@ pExpr at = Expr <$> pExprOrPattern True at
 
 -- | Parse an expression, as a concatenation of elements.
 pExprOrPattern :: Bool -> [Token] -> Parser TT [Exp TT]
-pExprOrPattern isExpresssion at = pure [] 
-       <|> ((:) <$> pElem isExpresssion at          <*> pExprOrPattern True at)
-       <|> ((:) <$> (TS <$> exact [ReservedOp DoubleColon] <*> pTypeExpr') <*> pure [])
-     -- TODO: not really correct: in (x :: X , y :: Z), all after the first :: will be a "type".
+pExprOrPattern isExpresssion at =
+  pure []
+  <|> ((:) <$> pElem isExpresssion at          <*> pExprOrPattern True at)
+  <|> ((:) <$> (TS <$> exact [ReservedOp DoubleColon] <*> pTypeExpr')
+       <*> pure [])
+     -- TODO: not really correct: in (x :: X , y :: Z), all after the
+     -- first :: will be a "type".
 
 pPattern = pExprOrPattern False recognizedSometimes
 
@@ -610,40 +640,54 @@ pExprElem = pElem True
 -- | Parse an "element" of an expression or a pattern.
 -- "at" is a list of symbols that, if found, should be considered errors.
 pElem :: Bool -> [Token] -> Parser TT (Exp TT)
-pElem isExpresssion at
-    = pCParen (pExprOrPattern isExpresssion (recognizedSometimes \\ [Special ','])) pEmpty -- might be a tuple, so accept commas as noise
-  <|> pCBrack (pExprOrPattern isExpresssion (recognizedSometimes \\ [ReservedOp DoubleDot, ReservedOp Pipe, ReservedOp LeftArrow, Special ','])) pEmpty -- list thing
-  <|> pCBrace (many $ pElem isExpresssion (recognizedSometimes \\ [ReservedOp Equal, Special ',', ReservedOp Pipe])) pEmpty -- record: TODO: improve
-  <|> (Yuck $ Enter "incorrectly placed block" $ pBlockOf (pExpr recognizedSometimes)) -- no error token, but the previous keyword will be one. (of, where, ...)
+pElem isExpresssion at =
+  pCParen (pExprOrPattern isExpresssion
+           -- might be a tuple, so accept commas as noise
+           (recognizedSometimes \\ [Special ','])) pEmpty
+  <|> pCBrack (pExprOrPattern isExpresssion
+               (recognizedSometimes \\ [ ReservedOp DoubleDot, ReservedOp Pipe
+                                       , ReservedOp LeftArrow
+                                       , Special ','])) pEmpty -- list thing
+  <|> pCBrace (many $ pElem isExpresssion
+               -- record: TODO: improve
+               (recognizedSometimes \\ [ ReservedOp Equal, Special ','
+                                       , ReservedOp Pipe])) pEmpty
+  <|> (Yuck $ Enter "incorrectly placed block" $
+        -- no error token, but the previous keyword will be one. (of, where, ...)
+         pBlockOf (pExpr recognizedSometimes))
   <|> (PError <$> recoverWith
        (sym $ flip elem $ isNoiseErr at) <*> errTok <*> pEmpty)
-  <|> (PAtom <$> sym (flip notElem (isNotNoise at)) <*> pEmpty)
-  <|> if isExpresssion then (pLet <|> pDo <|> pOf <|> pLambda) else empty
+  <|> (PAtom <$> sym (`notElem` isNotNoise at) <*> pEmpty)
+  <|> if isExpresssion then pLet <|> pDo <|> pOf <|> pLambda else empty
   -- TODO: support type expressions
 
 pTypeExpr at = many (pTypeElem at)
-pTypeExpr' = pTypeExpr (recognizedSometimes \\ [ReservedOp RightArrow, 
+pTypeExpr' = pTypeExpr (recognizedSometimes \\ [ReservedOp RightArrow,
                                                 ReservedOp DoubleRightArrow])
 
 pTypeElem :: [Token] -> Parser TT (Exp TT)
-pTypeElem at 
-    = pCParen (pTypeExpr (recognizedSometimes \\ [ReservedOp RightArrow,
-                                                  ReservedOp DoubleRightArrow,
-                                                  Special ','])) pEmpty -- might be a tuple, so accept commas as noise
+pTypeElem at
+    = pCParen (pTypeExpr (recognizedSometimes
+                          \\ [ ReservedOp RightArrow,
+                              ReservedOp DoubleRightArrow,
+                              -- might be a tuple, so accept commas as noise
+                              Special ','])) pEmpty
   <|> pCBrack pTypeExpr' pEmpty
   <|> pCBrace pTypeExpr' pEmpty -- TODO: this is an error: mark as such.
-  <|> (Yuck $ Enter "incorrectly placed block" $ pBlockOf (pExpr recognizedSometimes))
+  <|> (Yuck $ Enter "incorrectly placed block" $
+         pBlockOf (pExpr recognizedSometimes))
   <|> (PError <$> recoverWith
        (sym $ flip elem $ isNoiseErr at) <*> errTok <*> pEmpty)
-  <|> (PAtom <$> sym (flip notElem (isNotNoise at)) <*> pEmpty)
+  <|> (PAtom <$> sym (`notElem` isNotNoise at) <*> pEmpty)
 
--- | List of things that allways should be parsed as errors
+-- | List of things that always should be parsed as errors
 isNoiseErr :: [Token] -> [Token]
 isNoiseErr r = recoverableSymbols ++ r
 
 recoverableSymbols = recognizedSymbols \\ fmap Special "([{<>."
 -- We just don't recover opening symbols (only closing are "fixed").
--- Layout symbols "<>." are never recovered, because layout is constructed correctly.
+-- Layout symbols "<>." are never recovered, because layout is
+-- constructed correctly.
 
 -- | List of things that should not be parsed as noise
 isNotNoise :: [Token] -> [Token]
@@ -651,20 +695,20 @@ isNotNoise r = recognizedSymbols ++ r
 
 -- | These symbols are always properly recognized, and therefore they
 -- should never be accepted as "noise" inside expressions.
-recognizedSymbols = 
-    [ (Reserved Let)
-    , (Reserved In)
-    , (Reserved Do)
-    , (Reserved Of)
-    , (Reserved Class)
-    , (Reserved Instance)
-    , (Reserved Deriving)
-    , (Reserved Module)
-    , (Reserved Import)
-    , (Reserved Type)
-    , (Reserved Data)
-    , (Reserved NewType)
-    , (Reserved Where)] ++ fmap Special "()[]{}<>."
+recognizedSymbols =
+    [ Reserved Let
+    , Reserved In
+    , Reserved Do
+    , Reserved Of
+    , Reserved Class
+    , Reserved Instance
+    , Reserved Deriving
+    , Reserved Module
+    , Reserved Import
+    , Reserved Type
+    , Reserved Data
+    , Reserved NewType
+    , Reserved Where] ++ fmap Special "()[]{}<>."
 
 -- | Parse parenthesis, brackets and braces containing
 -- an expression followed by possible comments
@@ -693,12 +737,11 @@ pBrack = flip pCBrack pComments
 pEBrace p = Paren  <$> pCAtom [Special '{'] pEmpty
         <*> p <*> (recoverAtom <|> pCAtom [Special '}'] pComments)
 
--- | Create a special error token. (e.g. fill in where there is no correct token to parse)
--- Note that the position of the token has to be correct for correct computation of 
--- node spans.
+-- | Create a special error token. (e.g. fill in where there is no
+-- correct token to parse) Note that the position of the token has to
+-- be correct for correct computation of node spans.
 errTok = mkTok <$> curPos
    where curPos = tB <$> lookNext
          tB Nothing = maxBound
          tB (Just x) = tokBegin x
          mkTok p = Tok (Special '!') 0 (startPosn {posnOfs = p})
-
